@@ -26,25 +26,52 @@ pub fn trim_paf_rec_to_rgn(rgn: &bed::Region, paf: &PafRecord) -> Option<PafReco
 
     // index at the start of trimmed alignment
     trimmed_paf.t_st = cmp::max(rgn.st, paf.t_st);
-    //eprintln!("start found: {}", trimmed_paf.t_st);
-    let start_idx = paf.tpos_to_idx(trimmed_paf.t_st);
+    let start_idx = match paf.tpos_to_idx(trimmed_paf.t_st, true) {
+        Ok(idx) => idx,
+        Err(_) => panic!(
+            "\nProblem getting index in cigar:\n{}\n{}\n{}\n",
+            trimmed_paf.t_st, rgn, paf
+        ),
+    };
+    trimmed_paf.t_st = paf.tpos_aln[start_idx];
     trimmed_paf.q_st = paf.qpos_aln[start_idx];
 
     // index at the end of trimmed alignment
     trimmed_paf.t_en = cmp::min(rgn.en, paf.t_en);
     // the end index is not inclusive so minus 1
-    let end_idx = paf.tpos_to_idx(trimmed_paf.t_en - 1); // not inclusive on the end so -1
+    let end_idx = match paf.tpos_to_idx(trimmed_paf.t_en - 1, false) {
+        Ok(idx) => idx,
+        Err(error_idx) => panic!(
+            "\nProblem getting index in cigar:\n{}\n{:?}\n{}\n{}\n",
+            trimmed_paf.t_en - 1,
+            &paf.tpos_aln[error_idx - 1],
+            rgn,
+            paf
+        ),
+    };
+    trimmed_paf.t_en = paf.tpos_aln[end_idx];
     trimmed_paf.q_en = paf.qpos_aln[end_idx];
 
+    // if this is the case then the whole internal region is indel
+    // TODO check to make sure this is true
+    if start_idx > end_idx {
+        return None;
+    }
+
     // get the cigar opts
     trimmed_paf.cigar = PafRecord::collapse_long_cigar(&paf.subset_cigar(start_idx, end_idx));
+    // trimmed_paf.long_cigar = paf.subset_cigar(start_idx, end_idx);
 
     if paf.strand == '-' {
         std::mem::swap(&mut trimmed_paf.q_en, &mut trimmed_paf.q_st);
     }
     // make end index not inclusive
+    trimmed_paf.t_en += 1;
     trimmed_paf.q_en += 1;
+
+    // trim indels from start and end
     trimmed_paf.remove_trailing_indels();
+
     if trimmed_paf.cigar.len() == 0 {
         return None;
     }
@@ -56,14 +83,16 @@ pub fn trim_paf_rec_to_rgn(rgn: &bed::Region, paf: &PafRecord) -> Option<PafReco
         return None;
     }
 
+    // check that this is still a valid paf record
+    if let Err(e) = trimmed_paf.check_integrity() {
+        eprintln!("WARNING: {:?}", e);
+        return None;
+    };
+
     Some(trimmed_paf)
 }
 
-pub fn trim_help(rgn: &bed::Region, rec: &PafRecord) -> Option<PafRecord> {
-    trim_paf_rec_to_rgn(rgn, rec)
-}
-
-pub fn trim_help_2(name: &str, recs: &[PafRecord], rgns: &[bed::Region]) -> Vec<PafRecord> {
+pub fn trim_helper(name: &str, recs: &[PafRecord], rgns: &[bed::Region]) -> Vec<PafRecord> {
     let mut cur_recs: Vec<PafRecord> = recs
         .into_par_iter()
         .filter(|rec| rec.t_name == name)
@@ -84,7 +113,7 @@ pub fn trim_help_2(name: &str, recs: &[PafRecord], rgns: &[bed::Region]) -> Vec<
         .cartesian_product(cur_rgns) // make all pairwise combs
         .par_bridge()
         .filter(|(paf, rgn)| paf.paf_overlaps_rgn(rgn)) //filter to overlaping pairs
-        .filter_map(|(paf, rgn)| trim_help(rgn, paf))
+        .filter_map(|(paf, rgn)| trim_paf_rec_to_rgn(rgn, paf))
         .collect();
 
     cur_trimmed_paf
@@ -119,7 +148,7 @@ pub fn trim_paf_by_rgns(
             idx + 1,
             names.len()
         );
-        let mut tmp = trim_help_2(name, recs, rgns);
+        let mut tmp = trim_helper(name, recs, rgns);
         trimmed_paf.append(&mut tmp);
     }
     eprintln!();
@@ -152,6 +181,7 @@ pub fn break_paf_on_indels(paf: &PafRecord, break_length: u32) -> Vec<PafRecord>
                     st: pre_tpos,
                     en: cur_tpos,
                     id: paf.id.clone(),
+                    ..Default::default()
                 };
                 //rtn.push(trim_paf_rec_to_rgn(&rgn, paf));
                 if let Some(x) = trim_paf_rec_to_rgn(&rgn, paf) {
@@ -174,6 +204,7 @@ pub fn break_paf_on_indels(paf: &PafRecord, break_length: u32) -> Vec<PafRecord>
             st: pre_tpos,
             en: cur_tpos,
             id: paf.id.clone(),
+            ..Default::default()
         };
         if let Some(x) = trim_paf_rec_to_rgn(&rgn, paf) {
             rtn.push(x)
@@ -212,18 +243,21 @@ mod tests {
             st: 14,
             en: 15,
             id: "None".to_string(),
+            ..Default::default()
         };
         let rgn2 = Region {
             name: "T".to_string(),
             st: 14,
             en: 18,
             id: "".to_string(),
+            ..Default::default()
         };
         let rgn3 = Region {
             name: "T".to_string(),
             st: 12,
             en: 20,
             id: "".to_string(),
+            ..Default::default()
         };
         // test right extend
         let rgn4 = Region {
@@ -231,6 +265,7 @@ mod tests {
             st: 12,
             en: 30,
             id: "".to_string(),
+            ..Default::default()
         };
         // test left extend
         let rgn5 = Region {
@@ -238,6 +273,7 @@ mod tests {
             st: 5,
             en: 20,
             id: "".to_string(),
+            ..Default::default()
         };
         // test both extend
         let rgn6 = Region {
@@ -245,6 +281,7 @@ mod tests {
             st: 5,
             en: 30,
             id: "".to_string(),
+            ..Default::default()
         };
 
         let sts = vec![4, 7, 4, 4, 2, 2, 2, 2, 2, 2, 2, 2];
